"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PageEventCollector = void 0;
const selector_1 = require("./selector");
const element_1 = require("./element");
const serialize_1 = require("./serialize");
class PageEventCollector {
    constructor(options) {
        this._onDispose = [];
        this._attribute = options.attribute;
        this._pageIndex = options.pageIndex;
        this._sendEvent = options.sendEvent;
        this.collectEvents();
        console.debug('PageEventCollector: created', options);
    }
    dispose() {
        this._onDispose.forEach((d) => d());
        console.debug('PageEventCollector: disposed');
    }
    listen(eventName, handler) {
        document.addEventListener(eventName, handler, {
            capture: true,
            passive: true,
        });
        this._onDispose.push(() => document.removeEventListener(eventName, handler));
    }
    sendEvent(eventName, event, value) {
        const target = event.target;
        const isTargetVisible = element_1.isVisible(target, window.getComputedStyle(target));
        const elementEvent = {
            isTrusted: event.isTrusted && isTargetVisible,
            name: eventName,
            page: this._pageIndex,
            selector: selector_1.buildSelector({
                attribute: this._attribute,
                isClick: ['click', 'mousedown'].includes(eventName),
                target,
            }),
            target: serialize_1.nodeToDoc(target),
            time: Date.now(),
            value,
        };
        console.debug(`PageEventCollector: ${eventName} event`, event, event.target, 'recorded:', elementEvent);
        this._sendEvent(elementEvent);
    }
    collectEvents() {
        this.listen('mousedown', (event) => {
            if (event.button !== 0)
                return;
            const target = element_1.getClickableAncestor(event.target);
            this.sendEvent('mousedown', Object.assign(Object.assign({}, event), { target }));
        });
        this.listen('click', (event) => {
            if (event.button !== 0)
                return;
            const target = element_1.getClickableAncestor(event.target);
            this.sendEvent('click', Object.assign(Object.assign({}, event), { target }));
        });
        this.listen('input', (event) => {
            const target = event.target;
            if (target.tagName.toLowerCase() !== 'select')
                return;
            this.sendEvent('input', event, target.value);
        });
        this.listen('keydown', (event) => {
            this.sendEvent('keydown', event, event.key);
        });
        this.listen('keyup', (event) => {
            this.sendEvent('keyup', event, event.key);
        });
        this.listen('paste', (event) => {
            if (!event.clipboardData)
                return;
            const value = event.clipboardData.getData('text');
            this.sendEvent('paste', event, value);
        });
        this.listen('select', (event) => {
            const target = event.target;
            if (target.selectionStart !== 0 ||
                target.selectionEnd !== target.value.length) {
                return;
            }
            this.sendEvent('selectall', event);
        });
        this.collectScrollEvent();
    }
    collectScrollEvent() {
        let lastWheelEvent = null;
        this.listen('wheel', (ev) => (lastWheelEvent = ev));
        this.listen('scroll', (event) => {
            if (!lastWheelEvent || event.timeStamp - lastWheelEvent.timeStamp > 100) {
                console.debug('ignore non-wheel s:croll event', event);
                return;
            }
            let target = event.target;
            if (event.target === document || event.target === document.body) {
                target = (document.scrollingElement ||
                    document.documentElement);
            }
            const value = {
                x: target.scrollLeft,
                y: target.scrollTop,
            };
            this.sendEvent('scroll', Object.assign(Object.assign({}, event), { target }), value);
        });
    }
}
exports.PageEventCollector = PageEventCollector;
//# sourceMappingURL=PageEventCollector.js.map