"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSelectorParts = exports.buildCues = exports.buildCuesForElement = exports.buildTextCues = exports.buildCueValueForTag = exports.buildAttributeCues = void 0;
const attribute_1 = require("./attribute");
const isDynamic_1 = require("./isDynamic");
const DEFAULT_ATTRIBUTE = 'data-cy,data-e2e,data-qa,data-test,data-testid,/^qa-.*/';
const CSS_ATTRIBUTES = [
    'alt',
    'aria-label',
    'contenteditable',
    'for',
    'href',
    'name',
    'placeholder',
    'src',
    'title',
    'value',
];
const CueTypes = [
    ...CSS_ATTRIBUTES,
    'attribute',
    'class',
    'id',
    'tag',
    'text',
];
exports.buildAttributeCues = ({ attributes, element, level, useAttributeName, }) => {
    const cues = [];
    attributes.forEach((attribute) => {
        const attributeValuePair = attribute_1.getAttribute({ attribute, element });
        if (!attributeValuePair)
            return;
        const { name, value } = attributeValuePair;
        const type = (useAttributeName ? name : 'attribute');
        cues.push({ level, type, value: `[${name}="${value}"]` });
    });
    return cues;
};
exports.buildCueValueForTag = (element) => {
    const tagName = element.tagName.toLowerCase();
    if (!element.parentElement)
        return tagName;
    const siblings = element.parentElement.children;
    const sameTagSiblings = [];
    for (const sibling of siblings) {
        if (sibling.tagName.toLowerCase() === tagName) {
            sameTagSiblings.push(sibling);
        }
    }
    if (sameTagSiblings.length < 2) {
        return tagName;
    }
    const nthIndex = sameTagSiblings.indexOf(element) + 1;
    if (nthIndex === 1)
        return tagName;
    return `${tagName}:nth-of-type(${nthIndex})`;
};
exports.buildTextCues = ({ element, isClick, level, }) => {
    if (!isClick)
        return [];
    let text = element.innerText.trim();
    if (element instanceof HTMLInputElement &&
        ['button', 'submit'].includes(element.type)) {
        text = element.value;
    }
    if (!text ||
        text.length > 200 ||
        text.match(/[\n\r\t]+/) ||
        text.match(/[^\x20-\x7E]/g))
        return [];
    return [{ level, type: 'text', value: text }];
};
exports.buildCuesForElement = ({ attributes, element, isClick, level, }) => {
    const cues = [
        ...exports.buildAttributeCues({ attributes, element, level }),
        ...exports.buildAttributeCues({
            attributes: [...CSS_ATTRIBUTES],
            element,
            level,
            useAttributeName: true,
        }),
        ...exports.buildTextCues({ element, isClick, level }),
    ];
    element.classList.forEach((c) => {
        if (isDynamic_1.isDynamic(c))
            return;
        cues.push({ level, type: 'class', value: `.${c}` });
    });
    if (element.id && !isDynamic_1.isDynamic(element.id)) {
        cues.push({ level, type: 'id', value: `#${element.id}` });
    }
    cues.push({ level, type: 'tag', value: exports.buildCueValueForTag(element) });
    return cues;
};
exports.buildCues = ({ attribute, isClick, target }) => {
    const attributes = (attribute || DEFAULT_ATTRIBUTE).split(',');
    const cues = [];
    let element = target;
    let level = 0;
    while (element) {
        cues.push(...exports.buildCuesForElement({ attributes, element, isClick, level }));
        element = element.parentElement;
        level += 1;
    }
    return cues;
};
exports.buildSelectorParts = (cues) => {
    const levels = [...new Set(cues.map((cue) => cue.level))];
    levels.sort().reverse();
    const parts = [];
    levels.forEach((level) => {
        const cuesForLevel = cues.filter((cue) => cue.level === level);
        const textCues = cuesForLevel.filter((cue) => cue.type === 'text');
        if (textCues.length) {
            parts.push({ name: 'text', body: textCues[0].value });
            return;
        }
        cuesForLevel.sort((a, b) => {
            if (a.type === 'tag')
                return -1;
            if (b.type === 'tag')
                return 1;
            return 0;
        });
        const bodyValues = cuesForLevel.map((cue) => cue.value);
        parts.push({ name: 'css', body: bodyValues.join('') });
    });
    return parts;
};
//# sourceMappingURL=cues.js.map